@@ -40,6 +40,10 @@
  * 2002/03/01 - EvB - Made buildtree even more gross by having it delete
  * 		      items with noenc=1 from its source list. Removes memory
  * 		      leak though. Cleanup still needed.
+ * 2002/03/17 - EvB - Removed that hack; noenc=1 items are just left on the
+ * 		      source list now.
+ * 2002/04/03 - EvB - Good cleanup of buildtree(). It also leaves the source 
+ * 		      list intact now, so that you can re-use parts of it.
  */
 
 char metaops_id[] = "METAOPS - Copyright (C) 2001 Emile van Bergen.";
@@ -118,15 +122,15 @@ static void debug_fixed(META_ITEM *i, char *blk, META_ORD blksize)
 {
 	if (i->len_ofs + i->len_size > blksize) {
 		msg(F_RECV, L_DEBUG, "meta_decode: ERROR: Length field for "
-				     "%s/%ld/%s at 0x%04lx, length 0x%lx not "
-				     "within area, length 0x%04lx!\n", 
+				     "%s/%ld/%s at 0x%lx, length %ld not "
+				     "within block, length 0x%lx!\n", 
 		    i->spc->name, i->vnd, i->name,
 		    i->len_ofs, i->len_size, blksize);
 	} 
 	else {
 		msg(F_RECV, L_DEBUG, "meta_decode: ERROR: Field %s/%ld/%s at "
-				     "0x%04lx, length 0x%lx not within area, "
-				     "length 0x%04lx!\n",
+				     "0x%lx, length %ld not within block, "
+				     "length 0x%lx!\n",
 		    i->spc->name, i->vnd, i->name,
 		    i->val_ofs, getvallen(i, getskiplen(i, blk, blksize)), 
 		    blksize);
@@ -140,16 +144,16 @@ static void debug_attr(META_SPC *s, META_ITEM *i, char *blk, char *p,
 	META_ORD atr, vnd, skiplen, vallen;
 
 	if (p > blk + blksize) {
-		msg(F_RECV, L_DEBUG, "meta_decode: ERROR: Arrived at 0x%04lx "
-				     "in space %s which is beyond 0x%04lx!\n",
+		msg(F_RECV, L_DEBUG, "meta_decode: ERROR: Arrived at 0x%lx "
+				     "in space %s which is beyond 0x%lx!\n",
 		    p - blk, s->name, blksize);
 		return;
 	}
 
 	if (p + s->atr_ofs + s->atr_size > blk + blksize) {
 		msg(F_RECV, L_DEBUG, "meta_decode: ERROR: Attribute field in "
-				     "space %s at 0x%04lx, length 0x%lx not "
-				     "within area, length 0x%04lx!\n",
+				     "space %s at 0x%lx, length %ld not "
+				     "within area, length 0x%lx!\n",
 		    s->name, p - blk + s->atr_ofs, s->atr_size, blksize);
 		return;
 	}
@@ -157,8 +161,8 @@ static void debug_attr(META_SPC *s, META_ITEM *i, char *blk, char *p,
 	atr = getatrnr(s, p);
 	if (p + s->vnd_ofs + s->vnd_size > blk + blksize) {
 		msg(F_RECV, L_DEBUG, "meta_decode: ERROR: Vendor field in "
-				     "space %s at 0x%04lx, length 0x%lx not "
-				     "within area, length 0x%04lx!\n",
+				     "space %s at 0x%lx, length %ld not "
+				     "within area, length 0x%lx!\n",
 		    s->name, p - blk + s->vnd_ofs, s->vnd_size, blksize);
 		return;
 	}
@@ -173,10 +177,18 @@ static void debug_attr(META_SPC *s, META_ITEM *i, char *blk, char *p,
 
 	skiplen = getskiplen(i, p, blksize);
 	vallen = getvallen(i, skiplen); 
-	msg(F_RECV, L_DEBUG, "meta_decode: ERROR: Value for attribute %s/%ld/%s"
-			     " at 0x%04lx, length 0x%lx (of 0x%lx) not within "
-			     "area, length 0x%04lx!\n",
-	    s->name, vnd, atr, p - blk + i->val_ofs, vallen, skiplen, blksize);
+	if (skiplen < 1 || vallen < 0) {
+		msg(F_RECV, L_DEBUG, "meta_decode: ERROR: Invalid skip length "
+				     "(%ld) and/or value length (%ld) for "
+				     "attribute %s/%ld/%s at 0x%lx!\n",
+		    skiplen, vallen, s->name, vnd, i->name, p-blk+i->val_ofs);
+		return;
+	}
+
+	msg(F_RECV, L_DEBUG, "meta_decode: ERROR: Value for attribute "
+			     "%s/%ld/%s at 0x%lx, length %ld "
+			     "not within area, length 0x%lx!\n",
+	    s->name, vnd, i->name, p - blk + i->val_ofs, vallen, blksize);
 }
 
 
@@ -274,7 +286,8 @@ META_AV *meta_decode(META *m, META_SPC *s, META_ORD defvnd,
 		  (i = meta_getitembynr(m, s, atrnr, META_ORD_ERR)) ||
 		  (i = meta_getitembynr(m, s, META_ORD_ERR, vndnr)) ||
 		  (i = meta_getitembynr(m, s, META_ORD_ERR, META_ORD_ERR))) &&
-	    	 p + i->val_ofs + ((skiplen = getskiplen(i, p, blksize)), (vallen = getvallen(i, skiplen))) <= blk + blksize;
+	    	 p + i->val_ofs + ((skiplen = getskiplen(i, p, blksize)), (vallen = getvallen(i, skiplen))) <= blk + blksize &&
+		 skiplen > 0 && vallen >= 0;
 	    p += skiplen, i = 0) {
 
 		/* Add the data, unless we shouldn't decode it */
@@ -439,11 +452,34 @@ META_ORD meta_encode(META_SPC *s, char *blk, META_ORD blksize, META_AV *avlist,
 }
 
 
-/* Build a tree from a list, inserting encapsulating attributes where needed.
-   It's become a bit of an ugly routine, after adding the double-linked AV
-   list and the AV_DEL flag & noenc handling. Should be cleaned up - I know. */
+/* Utility function for buildtree. Skips deleted items on the source list,
+   allocates a new item, makes it a copy of the current item on the source
+   list and advances the source list to the next item. The AV_FREE_P is
+   cleared on the copy, so that we don't double free things. */ 
 
-void meta_buildtree(META_AV **srclst, META_AV **dstlst, META_SPC *groundspc)
+static META_AV *getcopy(META_AV **src)
+{
+	META_AV *av;
+
+	while(*src && (((*src)->flags & AV_DEL) || ((*src)->i && (*src)->i->noenc))) *src = (*src)->next;
+	if (!*src) return 0;
+
+	av = (META_AV *)malloc(sizeof(META_AV));
+	if (!av) { msg(F_MISC, L_ERR, "meta_buildtree: ERROR: No memory!\n"); return 0; }
+	memset(av, 0, sizeof(META_AV));
+	av->i = (*src)->i;
+	av->p = (*src)->p;
+	av->l = (*src)->l;
+	av->flags &= ~AV_FREE_P;
+
+	*src = (*src)->next;
+	return av;
+}
+
+
+/* Build a tree from a list, adding encapsulating attributes where needed. */
+
+void meta_buildtree(META_AV *src, META_AV **dstlst, META_SPC *groundspc)
 {
 	META_AV **dststk[BT_STKDEPTH];
 	META_SPC *spcstk[BT_STKDEPTH];
@@ -454,52 +490,52 @@ void meta_buildtree(META_AV **srclst, META_AV **dstlst, META_SPC *groundspc)
 	META_SPC *curspc;
 	int itmsp, dstsp;
 
-	if (!srclst || !dstlst || !groundspc) return;
+	/* Check parameters, init destination list and stack, get first item */
+
+	if (!dstlst) return; *dstlst = 0; if (!src || !groundspc) return;
 	itmsp = dstsp = 0; 
 	curspc = groundspc;
-	
-	/* Find first non-deleted item */
-	for(curav = *srclst; 
-	    curav && ((curav->flags & AV_DEL) || (curav->i && curav->i->noenc));
-	    curav = curav->next);
-	if (!curav) { *dstlst = 0; return; }
+	curav = getcopy(&src);
 
-	/* Remove it from the list */
-	meta_remav(srclst, 0, curav);
-
-	for(;;) {
+	while(curav) {
 
 		/* See if the current item fits in current space */
+
 		if (curav->i->spc == curspc) {
 
 			/* Yes, so add the item to the destination list */
+
 			*dstlst = curav; 
 			dstlst = &(curav->next); 
 
-			/* Skip deleted items and take next one from the list */
-			do { curav = curav->next; } while(curav && ((curav->flags & AV_DEL) || (curav->i && curav->i->noenc)));
-			*dstlst = 0; if (!curav) return;
-			meta_remav(srclst, 0, curav);
-
 			/* If this is a single attr. space, leave it now. */
+
 			while(curspc->single && dstsp) {
 				curspc = spcstk[--dstsp];
 				dstlst = dststk[dstsp];
 			}
+
+			/* Take next item from source list and continue */
+
+			curav = getcopy(&src);
 			continue;
 		}
 
 		/* No, so try to reach the current space from this item's space
 		   by backtracking encapsulating items */
+
 		for(curitem = curav->i; curitem && curitem->spc != curspc; ) {
 
-		    	/* Not there yet, push the enc. item onto item stack.
-			   Try to find one with a matching vendor nr. first. */
+		    	/* Not there yet. First try to find an encapsulating 
+			   item for the current item's space that matches the 
+			   current item's vendor nr. */
+
 			for(i = curitem->spc->enc_items;
 			    i && i->vnd != curav->i->vnd;
 			    i = i->next);
 
-			/* Couldn't match vendor, find the 'any vendor' item */
+			/* Couldn't match vendor, find enc. item allowing any */
+
 			if (!i) {
 				for(i = curitem->spc->enc_items;
 				    i && i->vnd != META_ORD_ERR;
@@ -507,37 +543,47 @@ void meta_buildtree(META_AV **srclst, META_AV **dstlst, META_SPC *groundspc)
 			}
 
 			/* Add the found item to the encapsulating item stack */
+
 			curitem = i;
 			itmstk[itmsp++] = curitem;
 		}
 
-		/* See if we found an encapsulation route. If not, go one 
-		   (or more) spaces back on the space stack and retry. */
+		/* If we didn't find an encapsulation route to the current
+		   space, go back one or more spaces on the destination stack
+		   and retry. */
+
 		if (!curitem) {
 
 			/* If we're not at the bottom yet, pop current
-			   destination list and space from the stack and remove
-			   all items from the item stack. If we _are_ at the
+			   destination list and space and remove all useless
+			   encapsulation items we found. If we _are_ at the
 			   bottom, we apparently cannot handle this item. */
+
 			if (dstsp) {
 
-				/* Empty stack of (useless) obtained items */
+				/* Empty stack of useless obtained items */
+
 				itmsp = 0;
 
-				/* Keep going back if the current space
-				   is a 'single attribute' space. The spaces
-				   on the stack got above us on it because the
-				   (encapsulating) items in those space brought
-				   us here. (How can I say this clearly? ;-)).
-				   In any case, all those spaces already have
-				   their single attribute present in them. */
+				/* Go back one space and keep soing so if the
+				   space we get is a 'single attribute' space,
+				   because we know that there is already an
+				   attribute present from the fact that we're
+				   here -- a place which could only be reached
+				   using an encapsulating item. So all spaces
+				   below us already have their single allowed
+				   attribute present. */
+
 				do {
 					curspc = spcstk[--dstsp];
 					dstlst = dststk[dstsp];
 				}
 				while(curspc->single && dstsp);
 
-				/* Retry now we're one or more levels back */
+				/* Try again to add the item, directly or via
+				   an encapsulation route, now we're at a lower
+				   space */
+
 				continue;
 			}
 
@@ -553,37 +599,37 @@ void meta_buildtree(META_AV **srclst, META_AV **dstlst, META_SPC *groundspc)
 		/* We did succeed. Now follow the enc. route that we found,
 		   adding the encapsulating items to the destination list,
 		   while saving the destination list and space on its stack. */
+
 		while(itmsp) {
 
 			/* Allocate new AV item */
+
 			newav = (META_AV *)malloc(sizeof(META_AV));
-			if (!newav) {
-				msg(F_MISC, L_ERR, "meta_buildtree: ERROR: "
-						   "could not allocate new AV "
-						   "item!\n");
-				return;
-			} 
+			if (!newav) { msg(F_MISC, L_ERR, "meta_buildtree: ERROR: No memory!\n"); return; }
+			memset(newav, 0, sizeof(META_AV));
 
 			/* Pop item for this AV from encapsulating item stack */
-			memset(newav, 0, sizeof(META_AV));
+
 			newav->i = itmstk[--itmsp];
 
 			/* Add this item to the current destination list */
+
 			*dstlst = newav;
 			dstlst = &(newav->next);
 
-			/* Save the destination list tail (this AV item's next
-			   pointer) and the current space */
+			/* Save the destination list append point (this AV 
+			   item's next field) and this AV item's space 
+			   on the destination stack */
+
 			dststk[dstsp] = dstlst;
 			spcstk[dstsp++] = curspc;
 
 			/* Set the destination list to this item's subitem 
 			   list and the current space to this item's subspace */
+
 			dstlst = &(newav->sub);
 			curspc = newav->i->subspace;
 		}
-
-		/* Now we can retry again. */
 	}
 }
 
@@ -1004,7 +1050,7 @@ ssize_t meta_avtomsg(META *m, META_AV *avlst,
 		}
 
 		/* Not hex encoded but as humanly readable as possible */
-		t = av->i ? av->i->val_type : MT_STRING;
+		t = av->i ? av->i->val_type : (av->p ? MT_STRING : MT_INTEGER);
 		switch(t) {
 
 		  case MT_INTEGER:
@@ -1030,6 +1076,7 @@ ssize_t meta_avtomsg(META *m, META_AV *avlst,
 		  case MT_STRING:
 		  default:
 			/* If actually ordinal, convert to raw first */
+			/* FIXME -- do we actually allow that case anywhere? */
 		  	s = av->p;
 			if (!s) { 
 				s = (char *)&tmpord; 
