@@ -88,7 +88,7 @@ META_AV *meta_decode(META *m, META_SPC *s, META_ORD defvnd,
 
 /* Encoding */
 
-void meta_buildtree(META_AV **srclst, META_AV **dstlst, META_SPC *groundspc);
+void meta_buildtree(META_AV *srclst, META_AV **dstlst, META_SPC *groundspc);
 
 META_ORD meta_encode(META_SPC *s, char *blk, META_ORD blksize, META_AV *avlst,
 		     META_ORD *retvnd);
