@@ -16,6 +16,8 @@ int main()
 	META_SPC *spc;
 	META_AV *avlist;
 
+	msg_init(-1, 0);
+
 	inplen = 20 + 6 + 6 + 9 + 8 + 6 + 0x16 + 0xe + 6 + 0xb;
 	input = "\x01\xaa\x00\x6c"
 	        "ABCDEFGHIJKLMNOP"
@@ -59,6 +61,8 @@ int main()
 		+ AVMSG_ADDTAB + AVMSG_ADDSPACES + AVMSG_ADDTYPE
 		+ AVMSG_ADDIGNORE, 0);
 	write(1, output, outlen);
+	meta_freeavlist(avlist);
+	meta_del(m);
 	free(output);
 
 	return 0;
