@@ -24,6 +24,7 @@ int main()
 	int n;
 #endif
 
+	msg_init(-1, 0);
 	msg_setthresh(F_RECV, L_DEBUG);
 	msg_setthresh(F_SEND, L_DEBUG);
 
@@ -73,11 +74,12 @@ int main()
 #endif
 
 	printf("\nDECODING TEST\n");
-	input = "\x01\xaa\x00\x61"
+	input = "\x01\xaa\x00\x63"
 		"ABCDEFGHIJKLMNOP"
 		"\x06\x06\x00\x00\x00\x02"
 		"\x07\x06\x00\x00\x00\x02"
 		"\x20\x09TestNas"
+		"\x01\x00"
 		"\x01\x08Myself"
 		"\x04\x06\xc0\xa8\x05\xfe"
 		"\x1a\x0e" "\x00\x00\x00\x09" "\x01\x08" "TEST 1"
@@ -91,7 +93,7 @@ int main()
 		return 1;
 	}
 
-	avlist = meta_decode(m, s, 0, input, 0x61, 0);
+	avlist = meta_decode(m, s, 0, input, 0x63, 0);
 	if (!avlist) {
 		msg(F_MISC, L_ERR, "main: ERROR: error decoding data!\n");
 		return 1;
@@ -104,7 +106,7 @@ int main()
 
 	printf("\nBUILDTREE TEST\n");
 
-	meta_buildtree(&avlist, &avdstlst, s);
+	meta_buildtree(avlist, &avdstlst, s);
 	meta_printavlist(m, avdstlst, 0);
 	meta_freeavlist(avlist);
 
