@@ -11,6 +11,7 @@
  *
  * History:
  * 2001/04/27 - EvB - Created
+ * 2002/04/23 - EvB - Sanitized conversion operators
  */
 
 char langcompile_id[] = "LANGCOMPILE - Copyright (C) 2001 Emile van Bergen.";
@@ -85,7 +86,7 @@ typedef struct stdop {
 /* The standard operators we know */
 
 
-#define STDOP_CNT	77
+#define STDOP_CNT	67
 
 static STDOP stdops[STDOP_CNT] = {
 	{ "halt",	 NON, 32, L,   ANY, NON,   INT, OP_HALT },
@@ -93,29 +94,21 @@ static STDOP stdops[STDOP_CNT] = {
 	{ "delall",	 NON, 32, L,   ANY, ANY,   INT, OP_DELALLAV },
 	{ "del",	 NON, 32, L,   ANY, ANY,   INT, OP_DELAV },
 	{ "moveall",	 NON, 32, L,   ANY, ANY,   INT, OP_MOVEALLAV },
-	{ "asraw",	 INT, 30, L,   ANY, NON,   STR, OP_ORD2RAW },
-	{ "asraw",	 IPA, 30, L,   ANY, NON,   STR, OP_ORD2RAW },
-	{ "asraw",	 DAT, 30, L,   ANY, NON,   STR, OP_ORD2RAW },
-	{ "asstr",	 INT, 30, L,   ANY, NON,   STR, OP_ORD2DEC },
-	{ "asstr",	 IPA, 30, L,   ANY, NON,   STR, OP_ORD2DQ },
-	{ "asstr",	 DAT, 30, L,   ANY, NON,   STR, OP_ORD2YMDHMS },
-	{ "asint",	 IPA, 30, L,   ANY, NON,   INT, OP_NOP },
-	{ "asint",	 STR, 30, L,   ANY, NON,   INT, OP_DEC2ORD },
-	{ "asip",	 INT, 30, L,   ANY, NON,   IPA, OP_NOP },
-	{ "asip",	 STR, 30, L,   ANY, NON,   IPA, OP_DQ2ORD },
-	{ "ashexint",	 STR, 30, L,   ANY, NON,   INT, OP_HEX2ORD },
-	{ "ashexip",	 STR, 30, L,   ANY, NON,   IPA, OP_HEX2ORD },
-	{ "ashex",	 STR, 30, L,   ANY, NON,   STR, OP_HEX },
-	{ "ashex",	 ANY, 30, L,   ANY, NON,   STR, OP_ORD2HEX },
-	{ "asymdhms",    STR, 30, L,   ANY, NON,   DAT, OP_YMDHMS2ORD },
-	{ "asymdhms",    INT, 30, L,   ANY, NON,   STR, OP_ORD2YMDHMS },
-	{ "asymdhms",    DAT, 30, L,   ANY, NON,   STR, OP_ORD2YMDHMS },
-	{ "asymd",	 STR, 30, L,   ANY, NON,   DAT, OP_YMD2ORD },
-	{ "asymd",       INT, 30, L,   ANY, NON,   STR, OP_ORD2YMD },
-	{ "asymd",       DAT, 30, L,   ANY, NON,   STR, OP_ORD2YMD },
-	{ "ashms",	 STR, 30, L,   ANY, NON,   DAT, OP_HMS2ORD },
-	{ "ashms",       INT, 30, L,   ANY, NON,   STR, OP_ORD2HMS },
-	{ "ashms",       DAT, 30, L,   ANY, NON,   STR, OP_ORD2HMS },
+	/* Typecasts / auto-conversion (uses autodetected format for ints) */
+	{ "toint",	 ANY, 30, L,   INT, NON,   INT, OP_NOP },
+	{ "toip",	 ANY, 30, L,   IPA, NON,   IPA, OP_NOP },
+	{ "todate",	 ANY, 30, L,   DAT, NON,   DAT, OP_NOP },
+	{ "tostr",	 ANY, 30, L,   STR, NON,   STR, OP_NOP },
+	/* Convert particular string format to int (oct/dec/hex/raw -> int) */
+	{ "fromoct",	 ANY, 30, L,   STR, NON,   INT, OP_OCTSTR2ORD },
+	{ "fromdec",	 ANY, 30, L,   STR, NON,   INT, OP_DECSTR2ORD },
+	{ "fromhex",	 ANY, 30, L,   STR, NON,   INT, OP_HEXSTR2ORD },
+	{ "fromraw",	 ANY, 30, L,   STR, NON,   INT, OP_RAWSTR2ORD },
+	/* Convert int to particular string format (int -> oct/dec/hex/raw) */
+	{ "tooct",	 ANY, 30, L,   INT, NON,   STR, OP_ORD2OCTSTR },
+	{ "todec",	 ANY, 30, L,   INT, NON,   STR, OP_ORD2DECSTR },
+	{ "tohex",	 ANY, 30, L,   INT, NON,   STR, OP_ORD2HEXSTR },
+	{ "toraw",	 ANY, 30, L,   INT, NON,   STR, OP_ORD2RAWSTR },
 	{ "~",		 NON, 28, R,   ANY, INT,   INT, OP_NOT },
 	{ "-",		 NON, 28, R,   ANY, INT,   INT, OP_NEG },
 	{ "!",		 NON, 28, R,   ANY, ANY,   INT, OP_NO },
@@ -125,6 +118,8 @@ static STDOP stdops[STDOP_CNT] = {
 	{ "/",		 IPA, 25, L,   ANY, INT,   IPA, OP_CIDRMASK },
 	{ "/",		 ANY, 25, L,   INT, INT,   INT, OP_DIV },
 	{ "%",		 ANY, 25, L,   INT, INT,   INT, OP_MOD },
+	{ "+",		 DAT, 24, L,   ANY, INT,   DAT, OP_ADD },
+	{ "+",		 IPA, 24, L,   ANY, INT,   IPA, OP_ADD },
 	{ "+",		 ANY, 24, L,   INT, INT,   INT, OP_ADD },
 	{ "-",		 ANY, 24, L,   INT, INT,   INT, OP_SUB },
 	{ ">>",		 ANY, 23, L,   INT, INT,   INT, OP_SHR },
@@ -135,8 +130,8 @@ static STDOP stdops[STDOP_CNT] = {
 	{ "^",		 STR, 22, L,   ANY, STR,   STR, OP_XORSTR },
 	{ "&",		 ANY, 21, L,   INT, INT,   INT, OP_AND },
 	{ "|",		 ANY, 20, L,   INT, INT,   INT, OP_OR },
-	{ "as",	 	 INT, 18, L,   ANY, STR,   STR, OP_ORD2DFMT },
-	{ "as",	 	 DAT, 18, L,   ANY, STR,   STR, OP_ORD2DFMT },
+	/* Convert to run-time specified format (only implemented for dates) */
+	{ "as", 	 ANY, 18, L,   DAT, STR,   STR, OP_ORD2DFMSTR },
 	{ "beforefirst", ANY, 16, L,   STR, STR,   STR, OP_BF },
 	{ "afterfirst",  ANY, 16, L,   STR, STR,   STR, OP_AF },
 	{ "beforelast",  ANY, 16, L,   STR, STR,   STR, OP_BL },
@@ -167,13 +162,6 @@ static STDOP stdops[STDOP_CNT] = {
 	{ ",",		 ANY,  1, L,   ANY, NON,   NON, OP_POP }
 	};
 
-/* Hrmpf. We have this funny problem that unary operators after a binary
-   operator with a higher precedence will give problems. Eg. 1 + !0 will break
-   and try to emit an 'add' as soon as the '!' is encountered. We could check 
-   all precedence breaks in that the context may not be NON if the req_context 
-   is something other than that, but for now I'll just give the unary operators 
-   a higher precedence - I feel a little uneasy about this thing though. */
-
 
 /*
  * FUNCTIONS
@@ -203,16 +191,16 @@ static int cvtop(int srctype, int dsttype)
 {
 	if (srctype == STR) {
 		switch(dsttype) {
-		  case INT: return OP_DEC2ORD;
-		  case IPA: return OP_DQ2ORD;
-		  case DAT: return OP_YMDHMS2ORD;
+		  case INT: return OP_INTSTR2ORD;
+		  case IPA: return OP_IPASTR2ORD;
+		  case DAT: return OP_DATSTR2ORD;
 		}
 	}
 	else if (dsttype == STR) {
 		switch(srctype) {
-		  case INT: return OP_ORD2DEC;
-		  case IPA: return OP_ORD2DQ;
-		  case DAT: return OP_ORD2YMDHMS;
+		  case INT: return OP_ORD2DECSTR;
+		  case IPA: return OP_ORD2IPASTR;
+		  case DAT: return OP_ORD2DATSTR;
 		}
 	}
 
@@ -415,9 +403,11 @@ static ssize_t reccompile(META *m, IFACE *ifaces, TEXT *source,
 		   They return the last evaluated term as the result. This
 		   makes constructs possible like ArgA || ArgB, returning
 		   ArgA if that has a true value, or ArgB if ArgA is false. */
-		if (c == '&' && I(1) == '&' && context != NON) { 
+		if (((c == '&' && I(1) == '&') ||
+		     (c == 'a' && I(2) == 'd' && I(1) == 'n')) && 
+		    context != NON) { 
 			if (precedence > 5) break;
-			S(2);
+			S(2 + (c == 'a'));
 			saved_o = o;
 			o->op = OP_JMPZ; o++;
 			o->op = OP_POP; o++; 
@@ -432,7 +422,8 @@ static ssize_t reccompile(META *m, IFACE *ifaces, TEXT *source,
 			ret += l + 2 * sizeof(INSN);
 			continue;
 		}
-		if (c == '|' && I(1) == '|' && context != NON) { 
+		if (((c == '|' && I(1) == '|') ||
+		     (c == 'o' && I(1) == 'r')) && context != NON) { 
 			if (precedence > 4) break;
 			S(2);
 			saved_o = o;
