@@ -157,6 +157,16 @@ IFACE *job_run(CONF *c, JOB *j)
 		if (msg_thresh[F_SEND] >= L_DEBUG)
 			meta_printavlist(c->m, j->vm->head[VM_REP], 0);
 
+		/* Build encapsulation tree and encode packet */
+		meta_buildtree(j->vm->head[VM_REP], &avtree, c->ds_rad_pkt);
+		if (!avtree) { msg(F_SEND, L_ERR, "job_run: ERROR: Could not "
+					     "build response tree!\n"); break; }
+		j->pktlen = meta_encode(c->ds_rad_pkt, j->pkt,
+					C_MAX_PKTSIZE, avtree, 0);
+		meta_freeavlist(avtree);
+		if (j->pktlen == -1) { msg(F_SEND, L_ERR, "job_run: ERROR: "
+				       "Could not encode response!\n"); break; }
+
 		/* Get the shared secret from the response list */
 		for(av = j->vm->head[VM_REP];
 		    av && (!av->i || av->i->nr != C_DI_SECRET);
@@ -166,17 +176,6 @@ IFACE *job_run(CONF *c, JOB *j)
 					   " to sign response!\n"); break;
 		}
 
-		/* Build encapsulation tree and encode packet */
-		avtree = 0;
-		meta_buildtree(&j->vm->head[VM_REP], &avtree, c->ds_rad_pkt);
-		if (!avtree) { msg(F_SEND, L_ERR, "job_run: ERROR: Could not "
-					     "build response tree!\n"); break; }
-		j->pktlen = meta_encode(c->ds_rad_pkt, j->pkt,
-					C_MAX_PKTSIZE, avtree, 0);
-		meta_freeavlist(avtree);
-		if (j->pktlen == -1) { msg(F_SEND, L_ERR, "job_run: ERROR: "
-				       "Could not encode response!\n"); break; }
-
 		/* Sign the packet using the secret described by 'av' */
 		md5_init(&mds);
 		md5_append(&mds, (unsigned char *)j->pkt, j->pktlen);
@@ -193,21 +192,18 @@ IFACE *job_run(CONF *c, JOB *j)
 
 	  case VM_ABORTED:		/* There was an 'abort' opcode */
 
-		msg(F_SEND, L_NOTICE, "job_run: Aborted by code - dropping "
-					"request.\n");
+		msg(F_SEND, L_NOTICE, "job_run: Aborted by code - dropping request.\n");
 		break;
 
 	  case VM_IFACETRAP:		/* There was an interface call */
 
 		ret = vm_getiface(j->vm);
-		msg(F_LANG, L_NOTICE, "job_run: Interface call: %s\n", 
-		    ret->name);
+		msg(F_LANG, L_NOTICE, "job_run: Interface call: %s\n", ret->name);
 	  	return ret;
 
 	  default:			/* The VM choked on the code */
 
-		msg(F_LANG, L_ERR, "job_run: Problem running compiled "
-				     "expression, error code %d!\n", n);
+		msg(F_LANG, L_ERR, "job_run: Invalid operation, error code %d - dropping request!\n", n);
 		break;
 	}
 	
