@@ -173,7 +173,9 @@ void sighandler(int sig)
 	sigaddset(&set, sig);
 	write(spipe[PIPE_W], &set, sizeof(set));
 
+#ifdef NO_SIGACTION
 	signal(sig, sighandler);
+#endif
 }
 
 
@@ -192,6 +194,9 @@ int main(int argc, char **argv)
 	PROC *p;
 	PROC_SET ps;
 	sigset_t sigs, oldsigs;
+#ifndef NO_SIGACTION
+	struct sigaction sa;
+#endif
 	fd_set rfds, wfds;
 	struct timeval tv, *tvp;
 	int status, sockhighest, reaping, n;
@@ -223,9 +228,16 @@ int main(int argc, char **argv)
 	}
 
 	/* Install handler */
+#ifdef NO_SIGACTION
 	signal(SIGHUP, sighandler); signal(SIGINT, sighandler);
-	signal(SIGQUIT, SIG_IGN);
-	signal(SIGTERM, sighandler); signal(SIGCHLD, sighandler);
+	signal(SIGQUIT, sighandler); signal(SIGTERM, sighandler); 
+	signal(SIGCHLD, sighandler);
+#else
+	sa.sa_flags = SA_NOCLDSTOP + SA_RESTART; sa.sa_handler = sighandler; 
+	sigaction(SIGHUP, &sa, 0); sigaction(SIGINT, &sa, 0);
+	sigaction(SIGQUIT, &sa, 0); sigaction(SIGTERM, &sa, 0);
+	sigaction(SIGCHLD, &sa, 0);
+#endif
 
 	/* Go to background */
 	if (!nobackgr) {
@@ -290,7 +302,7 @@ int main(int argc, char **argv)
 			sigfillset(&sigs); 
 			sigprocmask(SIG_BLOCK, &sigs, &oldsigs);
 				
-			/* Latch the set in pipe (if any) into 'sigs' */
+			/* Latch the set in the pipe (if any) into 'sigs' */
 			sigemptyset(&sigs);
 			n = read(spipe[PIPE_R], &sigs, sizeof(sigs));
 			if (n == -1 && errno != EAGAIN) { 
